package org.jxul.css;

import java.io.*;
import java.util.*;

import javax.swing.*;

import org.jxul.framework.XulDom;
import org.jxul.framework.XulFormatException;

import org.w3c.dom.*;

/**
 * A CSS StyleSheet
 */
public class StyleSheet
{
    public static final boolean debug;
    
    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }
    
    String name;
    Style [] styles;
    
    public String getName() { return name; }
    
    public Style [] getStyles()
    {
        return styles;
    }
    
    public StyleSheet(String name, Style [] styles)
    {
        this.name = name;
        this.styles = styles;
    }

    public void apply(XulDom target, Style s) throws XulFormatException
    {
        Hashtable peerMap = target.getPeerMap();
        String selector = s.getSelector();
            
        if (debug) System.out.println("Selector is: " + selector);
            
        String elementName = null;
        String className = null;
        String idName = null;
            
        // separated by a 'dot'?
        /* Note: in CSS a '.' operator is used to match 
            the value of the class attribute */
        int dotIdx = selector.indexOf('.');
        /* Note: in CSS a '#' operator is used to match 
            the value of the id attribute. */
        int poundIdx = selector.indexOf('#');
        if (dotIdx >=0)
        {
            elementName = selector.substring(0,dotIdx);
            if (debug) System.out.println("elementName: " + elementName);
            className = selector.substring(dotIdx + 1, selector.length());
            if (debug) System.out.println("className: " + className);
        }
        else if (poundIdx >= 0)
        {
            elementName = selector.substring(0,poundIdx);
            if (debug) System.out.println("elementName: " + elementName);
            idName = selector.substring(poundIdx + 1, selector.length());
            if (debug) System.out.println("idName: " + idName);                
        }
            
        if (elementName != null)
        {
            // select the applicable nodes
            Document doc = target.getXulDoc();
            
            Element root = doc.getDocumentElement();            
            
            NodeList elems = root.getElementsByTagName(elementName);
            int count = elems.getLength();
            if (debug) System.out.println("# of matching elements: " + count);
                
            if (className != null)
            {
                for (int j=0; j < count; j++)
                {
                    // class attribute?
                    Element elem = (Element) elems.item(j);
                        
                    Attr classAttr = elem.getAttributeNode(CssSyntax.ATTRIBUTE_CLASS);
                    if (classAttr != null)
                    {
                        String classVal = classAttr.getValue();
                        if (classVal.equals(className))
                        {
                            if (debug) System.out.println("Found : " + classVal);                                
                            // get the component
                            JComponent peer = (JComponent) peerMap.get(elem);
                                
                            if (peer != null)
                            {
                                if (debug) System.out.println("Found a matching peer!");
                                    
                                // apply the style
                                s.apply(peer);
                            }
                        } // end of if
                    } // end of if
                } // end of for (all children)                      
            } // end of if (classname != null)
            else if (idName != null)
            {
                for (int j=0; j < count; j++)
                {
                    // id attribute?
                    Element elem = (Element) elems.item(j);
                        
                    Attr idAttr = elem.getAttributeNode(CssSyntax.ATTRIBUTE_ID);
                    if (idAttr != null)
                    {
                        String idVal = idAttr.getValue();
                        if (idVal.equals(idName))
                        {
                            if (debug) System.out.println("Found : " + idVal);                                
                            // get the component
                            JComponent peer = (JComponent) peerMap.get(elem);
                                
                            if (peer != null)
                            {
                                if (debug) System.out.println("Found a matching peer!");
                                    
                                // apply the style
                                s.apply(peer);
                            }
                        } // end of if
                    } // end of if
                } // end of for (all children)                      
            }
            else
            {
                    
            }
        }
        else
        {
            // find all these classes?
                
        }
    }
    
    public void apply(XulDom target) throws XulFormatException
    {        
        // iterate through each style
        for (int i=0; i < styles.length; i++)
        {
            apply(target, styles[i]);
        }
    } // end of apply()
    
    public void applyPositionalStyles(XulDom target) throws XulFormatException
    {
        Hashtable peerMap = target.getPeerMap();
        
        System.out.println("TBD: applyPositionalStyles");
        
        // iterate through each style
        for (int i=0; i < styles.length; i++)
        {
            if (styles[i].isPositional())
            {
                System.out.println("applying " + styles[i].getSelector());
                apply(target, styles[i]);
            }
        }
    } // end of applyPositionalStyles
}
