/* TestRhino.java */
package org.jxul.framework;

import org.jxul.js.*;

import org.mozilla.javascript.*;
import java.awt.*;
import java.io.*;

public class TestRhino
{
    public static final boolean debug;
    
    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }
    

    static Scriptable scope;
    static boolean quitting;
    
    public static void main(String args[])
    {
        try
        {
            // Associate a new Context with this thread
            Context contxt = Context.enter();
 
            // Set version to JavaScript1.2 so that we get object-literal style
            // printing instead of "[object Object]"
            contxt.setLanguageVersion(Context.VERSION_1_2);
     
            long l1 = System.currentTimeMillis();
            scope = contxt.initStandardObjects(null);
            long l2 = System.currentTimeMillis();
            System.out.println("initStandardObjects() took " + (l2 - l1) + " milliseconds.");
            
            System.out.println("Returned : " + scope.getClass().getName());
            System.out.println("Classname: " + scope.getClassName());
            
            // add a document object to the scope that will
            // mirror the browser document object
            ScriptableObject.defineClass(scope, XulJsDocument.class);            
            System.out.println("document now part of the scope");
            
            // implicitly create a document object (and anything else we need)            
            contxt.evaluateString(scope, "document = new document();", "<cmd>", 1, null);            
            System.out.println("document object created in the scope");

            // test adding a function?
            contxt.evaluateString(scope, "function hideSprite()" +
		                                 "{" + 
		                                 "document.write(\"hideSprite called.\");" +
		                                 "} ", "<cmd>", 1, null);                        
            
            System.out.println("Function hideSprite added to the scope???");
            
            
            // Scope id's ? 
            Object [] ids = scope.getIds();
            if (ids != null)
            {
                System.out.println("ids.length: " + ids.length);
                for (int i=0; i < ids.length; i++)
                    System.out.println("Id[" + i + "]-" + ids);
            }
            else
                System.out.println("ids is null.");

            
            processSource(contxt, null);
            
            // Exit from the context.
            Context.exit();
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }

    public static void processSource(Context cx, String filename) {
        if (filename == null) {
            BufferedReader in = new BufferedReader
                (new InputStreamReader(System.in));
            String sourceName = "<stdin>";
            int lineno = 1;
            boolean hitEOF = false;
            do {
                int startline = lineno;
                System.err.print("js> ");
                System.err.flush();
                try {
                    String source = "";
                    // Collect lines of source to compile.
                    while(true) {
                        String newline;
                        newline = in.readLine();
                        if (newline == null) {
                            hitEOF = true;
                            break;
                        }
                        source = source + newline + "\n";
                        lineno++;
                        // Continue collecting as long as more lines
                        // are needed to complete the current
                        // statement.  stringIsCompilableUnit is also
                        // true if the source statement will result in
                        // any error other than one that might be
                        // resolved by appending more source.
                        if (cx.stringIsCompilableUnit(source))
                            break;
                    }
                    Object result = cx.evaluateString(scope, source,
                                                      sourceName, startline,
                                                      null);
                    if (result != cx.getUndefinedValue()) {
                        System.err.println(cx.toString(result));
                    }
                }
                catch (WrappedException we) {
                    // Some form of exception was caught by JavaScript and
                    // propagated up.
                    System.err.println(we.getWrappedException().toString());
                    we.printStackTrace();
                }
                catch (EvaluatorException ee) {
                    // Some form of JavaScript error.
                    System.err.println("js: " + ee.getMessage());
                }
                catch (JavaScriptException jse) {
                    // Some form of JavaScript error.
                    System.err.println("js: " + jse.getMessage());
                }
                catch (IOException ioe) {
                    System.err.println(ioe.toString());
                }
                if (quitting) {
                    // The user executed the quit() function.
                    break;
                }
            } while (!hitEOF);
            System.err.println();
        } else {
            FileReader in = null;
            try {
                in = new FileReader(filename);
            }
            catch (FileNotFoundException ex) {
                Context.reportError("Couldn't open file \"" + filename + "\".");
                return;
            }

            try {
                // Here we evalute the entire contents of the file as
                // a script. Text is printed only if the print() function
                // is called.
                cx.evaluateReader(scope, in, filename, 1, null);
            }
            catch (WrappedException we) {
                System.err.println(we.getWrappedException().toString());
                we.printStackTrace();
            }
            catch (EvaluatorException ee) {
                System.err.println("js: " + ee.getMessage());
            }
            catch (JavaScriptException jse) {
                System.err.println("js: " + jse.getMessage());
            }
            catch (IOException ioe) {
                System.err.println(ioe.toString());
            }
            finally {
                try {
                    in.close();
                }
                catch (IOException ioe) {
                    System.err.println(ioe.toString());
                }
            }
        }
        System.gc();
    }
}

