/* XulRadioGroup.java */
package org.jxul.framework;

import java.io.*;
import javax.swing.*;
import javax.swing.border.*;

import org.w3c.dom.*;

import org.jxul.util.*;
import org.jxul.css.*;

/**
 * A XUL Radio Group Box. 
 *
 * NOTE: although I have seen examples where this has a 
 *       orient property.  I think that is a mistake.
 *       A radio group is not a general purpose container in 
 *       most gui toolkits.
 */
public class XulRadioGroup extends JPanel
{
    public static final boolean debug;
    
    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }
    
    static int count;
    
    private Element radioGroupElement;
    // attributes
    String id;
    String orient;
    
    Style cssStyle;

    public XulRadioGroup(Element radioGroupElement, Style cssStyle) throws XulFormatException
    {
        this.radioGroupElement = radioGroupElement;
        this.cssStyle = cssStyle;
        count++;
        
        Attr idAttr = radioGroupElement.getAttributeNode(XulSyntax.RADIOGROUP_ATTR_ID);
        if (idAttr != null)
        {
            id = idAttr.getNodeValue();
        }
    
        Attr orientAttr = radioGroupElement.getAttributeNode(XulSyntax.RADIOGROUP_ATTR_ORIENT);
        if (orientAttr != null)
        {
            orient = orientAttr.getNodeValue();
            
            // set a Box Layout (if this is vertical or horizontal)
            if (orient.equals(XulSyntax.RADIOGROUP_ATTR_ORIENT_VERTICAL))
                this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
            else
                this.setLayout(new BoxLayout(this,BoxLayout.X_AXIS));                
        }

        // should have radio children
        if (radioGroupElement.hasChildNodes())
        {
            /* children should be a set of widget elements 
               or more boxes. */
            NodeList children = radioGroupElement.getChildNodes();
            
            ButtonGroup bg = new ButtonGroup();
            
            // iterate through each child, if a menu - create it
            int childCount = children.getLength();
            for (int i=0; i < childCount; i++)
            {
                // get the child node
                Node child = children.item(i);
                
                // switch on the type of the child
                
                if (child.getNodeName().equals(XulSyntax.ELEMENT_RADIOBUTTON))
                {
                    XulRadioButton radioButton = new XulRadioButton((Element)child);                    
                    bg.add(radioButton);
                    if (cssStyle != null)
                        cssStyle.apply(radioButton);
                    this.add(radioButton);
                }
            } // end of for (all children)
        } // end of if (has children)
    
    } // end of XulTitledBox Constructor
    
    /** for unit testing. */
    public static void main(String args[])
    {
        try
        {
            
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }    
}
