/* XulStack.java */
package org.jxul.framework;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

import org.w3c.dom.*;

import org.jxul.util.*;
import org.jxul.css.*;

/**
 * A XUL Stack. (implements transparent layers)
 */

public class XulStack extends JLayeredPane 
{
    public static final boolean debug;
    
    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }
    
    protected Element stackElement;
    // attributes
    String style;
    String id;
    
    Style cssStyle;
    Hashtable peerMap;
    Integer currentLayer;
    int layerCount = 1;
    
    
    public XulStack(Element stackElement, Hashtable peerMap) throws XulFormatException
    {
 
        this.stackElement = stackElement;
        this.peerMap = peerMap;
            
       
         //This is tough. How do we get the preferred
         //size of this? It should probably be an 
         //attribute I guess. 
         setPreferredSize(new Dimension(350,200));
         processChildren();      
           
           
    } // end of XulMenuBar Constructor

    public void processChildren() throws XulFormatException
    {
        if (stackElement != null)
            processChildren(stackElement);  
            
    }

    public void processChildren(Element theElement) throws XulFormatException
    {
        // should have widget children
        JComponent jcomp = null; 
        if (theElement.hasChildNodes())
        {
            /* children should be a set of widget elements 
               or more boxes. */
            NodeList children = theElement.getChildNodes();
            
            // iterate through each child, if a menu - create it
            int childCount = children.getLength();
            for (int i=0; i < childCount; i++)
            {
                // set the currentLayer
                currentLayer = new Integer(layerCount++);
                
                // get the child node
                Node child = children.item(i);
                if (debug)
                    System.out.println("Child[" + i + "]" +
                    child.getNodeName() + "-" + child.getNodeValue());
 
                // switch on the type of the child
                String childName = child.getNodeName();
                
                if (debug) System.out.println("childName: " + childName);
                
                if (childName.equals(XulSyntax.ELEMENT_BUTTON))
                {
                    XulButton button = new XulButton((Element)child);
                    if (cssStyle != null)
                        cssStyle.apply(button);
                        
                    // add to peerMap
                    peerMap.put(child, button);
                    jcomp = (JComponent)button;
                }
                else if (childName.equals(XulSyntax.ELEMENT_CHECKBOX))
                {
                    XulCheckBox checkBox = new XulCheckBox((Element)child);
                    if (cssStyle != null)
                        cssStyle.apply(checkBox);

                    // add to peerMap
                    peerMap.put(child, checkBox);
                    jcomp = (JComponent)checkBox;
                }
                else if (childName.equals(XulSyntax.ELEMENT_RADIOGROUP))
                {
                    XulRadioGroup radioGroup = new XulRadioGroup((Element)child, cssStyle);
                    if (cssStyle != null)
                        cssStyle.apply(radioGroup);
             
                    // add to peerMap
                    peerMap.put(child, radioGroup);
                    jcomp = (JComponent)radioGroup;
               
                }
                else if (childName.equals(XulSyntax.ELEMENT_TEXT))
                {
                    XulText text = new XulText((Element)child);
                    

                    // add to peerMap
                    peerMap.put(child, text);
                    jcomp = (JComponent)text;
                } 
		        else if (childName.equals(XulSyntax.ELEMENT_LABEL))
                {
                    XulLabel label = new XulLabel((Element)child);

                    // add to peerMap
                    peerMap.put(child, label);
                    jcomp = (JComponent)label;
                }                              
                else if (childName.equals(XulSyntax.ELEMENT_TABBOX))
                {
                    // create a XulTabBox
                    XulTabBox tabBox = new XulTabBox((Element)child, peerMap);
                        
      
                    // add to peerMap
                    peerMap.put(child, tabBox);
                    jcomp = (JComponent)tabBox;
                }
                else if (childName.equals(XulSyntax.ELEMENT_HBOX))
                {
                    // create a XulHBox
                    XulHBox hbox = new XulHBox((Element)child, peerMap);
                        
            
                    // add to peerMap
                    peerMap.put(child, hbox);
                    jcomp = (JComponent)hbox;
                }
                else if (childName.equals(XulSyntax.ELEMENT_VBOX))
                {
                    // create a XulVBox
                    XulVBox vbox = new XulVBox((Element)child, peerMap);
                        
                    // add to peerMap
                    peerMap.put(child, vbox);
          			jcomp = (JComponent)jcomp;
                }
                else if (childName.equals(XulSyntax.ELEMENT_TABBOX))
                {
                    // create a XulTabBox
                    XulTabBox tabBox = new XulTabBox((Element)child, peerMap);
        
                    // add to peerMap
                    peerMap.put(child, tabBox);
                    jcomp = (JComponent)tabBox;
                }
                else if (childName.equals(XulSyntax.ELEMENT_IMAGE))
                {
                    // create a XulImage
                    XulImage image = new XulImage((Element)child);
               
                    // add to peerMap
                    peerMap.put(child, image);
                    jcomp = (JComponent)image;
                }
                else if (childName.equals(XulSyntax.ELEMENT_GRID))
                {
                    System.out.println("Grid @ Layer: " + currentLayer);
                    
                    // create a XulImage
                    XulGrid grid = new XulGrid((Element)child, peerMap);
                        
          
                    peerMap.put(child, grid);
                    jcomp = (JComponent)grid;
                
                }
                else if (childName.equals(XulSyntax.ELEMENT_BULLETIN_BOARD))
                {
                    if (debug) System.out.println("Adding a Bulletin Board");
                    System.out.println("BB @ Layer: " + currentLayer);
                    
                    // create a XulBulletinBoard
                    XulBulletinBoard bb = new XulBulletinBoard((Element)child, peerMap);
                    
                    // add to peerMap
                    peerMap.put(child, bb);
        			jcomp = (JComponent)bb;
                }
                
                else
                {
                    System.out.println("*TBD*:Unhandled XUL element: " + childName);                        
                }
                if (jcomp != null)
                {
                	//For each child, we'll need to specifically do sizes
                	//since this is all in layers!\
                	Dimension size = jcomp.getPreferredSize();
                	jcomp.setBounds(0, 0, size.width,size.height);
                	this.add(jcomp, currentLayer);
                	jcomp = null;
            	}
                
            } // end of for (all children)
        } // end of if (children exist)               
    }
    
    /** for unit testing. */
    public static void main(String args[])
    {
        try
        {
            
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}
