/* XulTreeHead.java */
package org.jxul.framework;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import org.jxul.util.*;

/**
 * A XUL Treehead, which is simply
 * something that returns columnNames &
 * & style to a JTree
 */
public class XulTreeHead 
{
    public static final boolean debug;
    
    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }

    //a <treehead> element has column names. 
    private Vector m_colnames;
  
    private Element treeheadElement;
    // attributes
    String _class;
    String flex;
    String id;
    String style;

    Hashtable peerMap;

    public XulTreeHead(Element treeheadElement, Hashtable peerMap) throws XulFormatException
    {
        this.treeheadElement = treeheadElement; 
	this.peerMap = peerMap;

	Attr styleAttr = treeheadElement.getAttributeNode(XulSyntax.ATTR_STYLE);
        Attr _classAttr = treeheadElement.getAttributeNode(XulSyntax.ATTR_CLASS);
        Attr flexAttr = treeheadElement.getAttributeNode(XulSyntax.ATTR_FLEX);
        Attr idAttr = treeheadElement.getAttributeNode(XulSyntax.ATTR_ID);
       
        processChildren();
    } // end of XulTreeHead Constructor
        
    public void processChildren() throws XulFormatException
    {
        if (treeheadElement.hasChildNodes())
        {
            NodeList children = treeheadElement.getChildNodes();
         
            int childCount = children.getLength();
            for (int i=0; i < childCount; i++)
            {
                // get the child node
                Node child = children.item(i);
                String childName = child.getNodeName(); 
                if (debug)
                    System.out.println("Child[" + i + "]" +
                    childName + "-" + child.getNodeValue());
 
                // switch on the type of the child
               
		if (childName.equals(XulSyntax.ELEMENT_TREEROW))
		{
			XulTreeRow tr = new XulTreeRow((Element)child, peerMap);
			peerMap.put(tr, child);
			setColumnNames(tr.getRowValues());	
		}
		else
                {
			if (debug)
				System.out.println("*TBD - " + childName);
                }
            } // end of for (all children)
        } // end of if (children exist)                       
    }

    private void setColumnNames(Vector cols)
    {
	m_colnames = cols;	
    }

    //needed by treetable.XulTreeModel
    public Vector getColumnNames()
    {
	return m_colnames;
    }

    /** for unit testing. */
    public static void main(String args[])
    {
        try
        {
            
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}
