// XulJsDocument.java 
package org.jxul.js;

import org.jxul.util.*;
import org.jxul.framework.*;

import org.mozilla.javascript.*;
import org.w3c.dom.*;

import java.awt.*;
import javax.swing.*;
import java.util.*;

public class XulJsDocument extends ScriptableObject
{
    public static final boolean debug;
    
    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }    
        
    static XulDom xdom;
    
    // properties see pg 259 JavaScript Definitive Guide
    
    // property bgColor
    Color bgColor;
    
    // The method jsGet_bgColor defines the bgColor property. 
    public Color jsGet_bgColor() { return bgColor; }

    
    // create a javascript function to mirror - getElementById
    // *** getElementById() is part of DOM Level 2
    // it returns an Element
    public static Object jsFunction_getElementById(Context cx, Scriptable thisObj,
                                                   Object[] args, Function funObj)
    {        
        if (debug) 
        {
            System.out.println("function: getElementById");
    
            // just expecting a name
            for (int i=0; i < args.length; i++)
                System.out.println(args[i]);
        }
        
        String id = null;
        
        if (args.length > 0)
            id = (String) args[0];
        else
            return null;
            
        Document doc = xdom.getXulDoc();
        Element root = doc.getDocumentElement();
            
        Element target = DomUtil.findElementWithAttribute(root, "id", id);
        
        if (target != null)
        {
            // get the peer
            Hashtable peerMap = xdom.getPeerMap();
            JComponent peer = (JComponent) peerMap.get(target);
            if (peer != null)   
            {
                //XulElement xelem = new XulElement(target, peer);
                Scriptable scope = ScriptableObject.getTopLevelScope(thisObj);
                Scriptable result;
                Object [] xeArgs = { target, peer };
                try {
                    result = cx.newObject(scope, "XulJsElement", xeArgs);
                } catch (PropertyException e) 
                  {
                    throw Context.reportRuntimeError(e.getMessage());
                  } 
                  catch (NotAFunctionException e) 
                  {
                    throw Context.reportRuntimeError(e.getMessage());
                  }
                  catch (JavaScriptException e) 
                  {
                    throw Context.reportRuntimeError(e.getMessage());
                  }
                //return xelem;
		System.out.println("About to return the result ->" + result);
	
                return result;
            }
        }
        else
            System.out.println("no elements with id of " + id);
            
        return null; 
    }

    // create a javascript function to mirror - getElementById
    public static void jsFunction_write(Context cx, Scriptable thisObj,
                                            Object[] args, Function funObj)
    {
        for (int i=0; i < args.length; i++)
            System.out.print(args[i]);
        System.out.println();                    
    }

    // Method jsConstructor defines the JavaScript constructor 
    public void jsConstructor() 
    { 
        //System.out.println("JS constructor for XulJsDocument");    
    }
    
    public XulJsDocument() 
    {
        // when created by the javascript engine
        // will need to go out and "attach" to the
        // XUL DOM
    }
    
    public void setXulDom(XulDom env)
    {
        xdom = env;    
    }
    
    public String getClassName()
    {
        return "document"; 
    }
}
