package org.jxul.util;

import java.util.*;
import java.io.*;

import java.util.Vector;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

/**
 * The <b>StringUtil</b> class consists of static methods which provide
 * simple interfaces to manipulate strings.
 *
 * @author mcd
 */

public class StringUtil
{
    public static final boolean debug;
    
    static
    {
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }   
    
   /**
    * Replaces <i>limit</i> occurrences of <i>target</i> with <i>replace</i>
    * in <i>source</i>.
    *
    * @param	source	the string to start with
    * @param	target	the pattern to replace
    * @param	replace	the new value
    * @param	limit	the maximumn times to replace, or 0 for all
    *
    * @return a modified version of the source string
    */

   public static String replaceString (String source,
                                       String target,
                                       String replace,
                                       int limit)
      {
         String result = source;
         int len = target.length();
         int pos = result.indexOf (target);
         int count = 0;

         while ((pos >= 0) && ((limit <= 0) || (count < limit)))
         {
            result = result.substring (0, pos) + replace +
               result.substring (pos + len, result.length());
            count++;
            pos = result.indexOf (target, pos + replace.length());
         }

         return (result);
      }

   /**
    * Return a new Vector based on the input array.
    * @param data : The array to convert.
    */
   public static Vector arrayToVector(String data[])
      {
         Vector result = new Vector(data.length);
         for (int i = 0; i < data.length; i++)
            result.addElement(data[i]);
         return(result);
      }
   /**
    * Converts a Vector to a String array.
    * @param v : The vector to convert.
    * @return String[] : String[] or null.
    */
   public static String[] vectorToArray(Vector v)
      {
         String result[] = null;

         if (v != null)
         {
            result = new String[v.size()];
            for (int i = 0; i < v.size(); i++)
            {
               result[i] = (String)v.elementAt(i);
            }
         }
         return(result);
      }

   public static String[] delimetedToArray(String delim, String source)
      {
         StringTokenizer st = new StringTokenizer(source, delim);
         String newarray[] = new String[st.countTokens()];

         for (int i = 0; i < st.countTokens(); i++)
         {
            try
	    {
               newarray[i] = (st.nextToken()).trim();
	    } catch (NoSuchElementException nse)
            {
               nse.printStackTrace();
               return(newarray);
            }
         }
         return(newarray);
      }


   /** counts # of occurrences of a character in a string.
    *  @author MCD
    */
   public static int countOccurences(String s, char c)
      {
         int iCount=0;
         int iLength = s.length();
         int iMark=0;

         while (iMark < iLength)
         {
            int idx = s.indexOf(c, iMark);
            if (idx >= 0)
            {
               iCount++;
               iMark = idx + 1;
            }
            else
               break;
         }

         return iCount;
      }

   public static String stripPound(String s)
      {
         if (s.startsWith("#"))
            return s.substring(1,s.length());
         return s;
      }

   public static String trimQuotes(String s)
      {
         int iFirstIdx = s.indexOf('"');
         int iLastIdx = s.lastIndexOf('"');
         if (iFirstIdx >= 0 && iLastIdx >= 0)
            return s.substring(iFirstIdx + 1, iLastIdx);
         else
            return s;
      }

   // check if a string already exists in an array of Strings
   public static boolean stringExists(String s, String [] sNames,
                                      boolean ignoreCase)
      {
         boolean bStat = false;
         if (s == null || sNames == null)
            return bStat;

         for (int i=0; i < sNames.length; i++)
         {
            if ((ignoreCase? s.equalsIgnoreCase(sNames[i]) :
                 s.equals(sNames[i])))
            {
               bStat = true;
               break;
            }
         }
         return bStat;
      }

   // yank the file Extension
   public static String getFileExtension(String sFileName)
      {
         // find the dot
         int iDotIdx = sFileName.indexOf(".");
         if (iDotIdx < 0)
            return null;
         return sFileName.substring(iDotIdx + 1, sFileName.length());
      }

   // extract the Nth field in a delimited String, n starts at 1
   public static String extractNthField(int n, String sBuf, String delimiter)
      {
         String sOutField = null;
         StringTokenizer st = new StringTokenizer(sBuf, delimiter);
         if (st.countTokens() > n)
         {
            for (int i=0; i < n; i++)
               sOutField = st.nextToken();
         }

         return sOutField;
      }

   // replace a null string with some string
   public static String replaceNull(String s, String pattern)
      {
         if (s == null)
            return pattern;
         return s;
      }


   // 
   public static String[] copyTokensToArray(StringTokenizer st)
      {
         String tokens[] = new String[st.countTokens()];
         int i = 0;
         while(st.hasMoreTokens())
         {
            tokens[i] = st.nextToken();
            i++;
         }
         return tokens;
      }


   public static boolean isRequiredLength(int tokenLength,  int reqLength)
      {
         if (tokenLength == reqLength)
            return true;
         else
            return false;
      }


   public static boolean isRequiredLength(int tokenLength, int minLength,
                                          int maxLength)
      {
         if (tokenLength >= minLength && tokenLength <= maxLength)
            return true;
         else
            return false;
      }

   public static boolean isStringNumeric(String token)
      {
         if (token != null && token.length() > 0)
         {
            for (int i=0; i<token.length();i++)
            {
               if (!Character.isDigit(token.charAt(i)))
                  return false;
            }
         }
         else
            return false;

         return true;
      }


      /**
       *  Tokenize a String to an array of tokens.
       *  Uses StringTokenizer.
       */
      public static String[] tokenize(String input)
      {
         Vector v = new Vector();
         StringTokenizer t = new StringTokenizer(input);
         String cmd[];

         while (t.hasMoreTokens())
	    v.addElement(t.nextToken());
         cmd = new String[v.size()];
         v.copyInto(cmd);

         return cmd;
      }

      /**
       *
       */
      public static String [] tokenize(String input, String delimiter)
      {
         Vector v = new Vector();
         StringTokenizer t = new StringTokenizer(input, delimiter);
         String cmd[];

         while (t.hasMoreTokens())
	    v.addElement(t.nextToken());
         cmd = new String[v.size()];
         v.copyInto(cmd);

         return cmd;        
      }

      /**
       *
       */
      
      public static String [] tokenize(String input, String delimiter, boolean delimiterAsGroup)
      {
         Vector v = new Vector();

         String cmd[] = null;

         if (!delimiterAsGroup)
         {
                StringTokenizer t = new StringTokenizer(input, delimiter);
                while (t.hasMoreTokens())
                        v.addElement(t.nextToken());
        }
        else
        {
                int start = 0;
                int end = input.length();

                while (start < end)
                {
                        int delimIdx = input.indexOf(delimiter,start);
                        if (delimIdx < 0)
                        {
                                String tok = input.substring(start);
                                v.addElement(tok);
                                start = end;
                        }
                        else
                        {
                                String tok = input.substring(start, delimIdx);
                                v.addElement(tok);
                                start = delimIdx + delimiter.length();
                        }
                }

        }

        cmd = new String[v.size()];
        v.copyInto(cmd);
        return cmd;
      }

   // **** static variable of StringUtil. Used in uniqueId()
   static long lCount;

   /**
    * Generates a unique id.
    * pattern is optional.
    * set iMaxLength to -1 if there is NO max length.
    */
   public static String uniqueId(String sPattern, int iMaxLength)
    {
        return uniqueId(sPattern, iMaxLength, false);
    }
   
   /**
    * Generates a unique id.
    * pattern is optional.
    * sequence makes the third token a 1 up number.
    * set iMaxLength to -1 if there is NO max length.
    */
   public static String uniqueId(String sPattern, int iMaxLength,
                                 boolean sequence)
      {
         class Counter
         {
            public Counter() { StringUtil.lCount++; }
         }

         StringBuffer sb = new StringBuffer();
         sb.append("ID.");
         Counter tmp = new Counter();
         long l = (sequence)? StringUtil.lCount : 
                              tmp.hashCode() + StringUtil.lCount;
         sb.append(l);
         sb.append(".");
         if (sPattern != null)
            sb.append(sPattern + ".");
         sb.append(new StringBuffer("" + System.currentTimeMillis()).reverse());
         sb.append(".");
         sb.append(System.getProperty("user.name") + ".");
         if (iMaxLength > 0 && sb.length() > iMaxLength)
            sb.setLength(iMaxLength);

         return sb.toString();
      }
      
    // if a file exists, produce the filename of the "next version"
    public static String nextVersion(String fileName)
    {
        String nextFileName = null;
        File f = new File(fileName);
        if (f.exists())
        {
            // find next version #, version #'s just tacked on end
            int versionNum = 1;
            String tmpFileName = fileName + versionNum;
            while (new File(tmpFileName).exists())
            {
                versionNum++;
                tmpFileName = fileName + versionNum; 
            }
            nextFileName = tmpFileName;
        }
        else
            nextFileName = fileName;
        return nextFileName;
    }

    // if a file exists, produce the filename of the "next version"
    // Prepend the counter if prepend is true
    // NOTE: FIX THIS - this code assumes, the file is in the current
    // directory -- this is not a problem for the above version. *****
    public static String nextVersion(String fileName, boolean prepend)
    {
        String nextFileName = null;
        File f = new File(fileName);
        if (f.exists())
        {
            // find next version #, version #'s just tacked on end
            int versionNum = 1;
            String tmpFileName = fileName + versionNum;
            while (new File(tmpFileName).exists())
            {
                versionNum++;
                if (prepend)
                    tmpFileName = versionNum + "_" + fileName;
                else
                    tmpFileName = fileName + versionNum; 
            }
            nextFileName = tmpFileName;
        }
        else
            nextFileName = fileName;
        return nextFileName;
    }

    // if a file exists, produce the filename of the "next version"
    // does not assume path is attached.  Returns just a filename.
    public static String nextVersion(String path, String fileName, boolean prepend)
    {
        String nextFileName = null;
        File f = new File(path + File.separator + fileName);
        if (f.exists())
        {
            // find next version #, version #'s just tacked on end
            int versionNum = 1;
            String tmpFileName = (prepend) ? versionNum + "_" + fileName : fileName + versionNum;
            while (new File(path + File.separator + tmpFileName).exists())
            {
                versionNum++;
                tmpFileName = (prepend) ? versionNum + "_" + fileName : fileName + versionNum; 
            }
            nextFileName = tmpFileName;
        }
        else
            nextFileName = fileName;
        return nextFileName;
    }

    // if a file exists, produce the filename of the "next version"
    // Note: this version preserves the extension
    // extension includes the dot.
    public static String nextVersion(String fileName, String extension)
    {
        int dotidx = fileName.indexOf('.');
        String truncatedFileName = null;
        if (dotidx >= 0)
            truncatedFileName = fileName.substring(0,dotidx);
        else
            truncatedFileName = fileName;
            
        String nextFileName = null;
        File f = new File(fileName);
        if (f.exists())
        {
            // find next version #, version #'s just tacked on end
            int versionNum = 1;
            String tmpFileName = truncatedFileName + versionNum + extension;
            while (new File(tmpFileName).exists())
            {
                versionNum++;
                tmpFileName = truncatedFileName + versionNum + extension; 
            }
            nextFileName = tmpFileName;
        }
        else
            nextFileName = fileName;
        return nextFileName;
    }
    
    public static String persistentSequenceNumber(String filename, int start) throws IOException
    {
        return persistentSequenceNumber(filename, start, -1);    
    }    
    
    public static String persistentSequenceNumber(String filename, int start, int max) throws IOException
    {
        // check if file exists
        int current = 0;
        File f = new File(filename);
        RandomAccessFile raf = null;
        if (f.exists())
        {
            // get current out of file
            raf = new RandomAccessFile(f,"rw");
            current = raf.readInt();
        }
        else
        {
            current = start;
        }
        
        // add 1 to current or rollover
        if (max != -1 && current == max)
        {
            current = start;
        }
        else
        {
            if (raf != null) // first time?
                current++;
        }
        
        // store new number in a file
        if (raf == null)
            raf = new RandomAccessFile(f,"rw");
        
        raf.seek(0); // go back to beginning
        raf.writeInt(current); // write count
        raf.close();
        
        // convert to string
        return "" + current;        
    }
    
    static String pad(String inString, int desiredLength, char padChar)
    {
        if (inString.length() >= desiredLength)
            return inString;
            
        StringBuffer newStr = new StringBuffer(inString);
        for (int i=inString.length(); i < desiredLength; i++)
            newStr.append(padChar);
            
        return newStr.toString();
    }

    static String prepend(String inString, int desiredLength, char padChar)
    {
        if (inString.length() >= desiredLength)
            return inString;
            
        StringBuffer newStr = new StringBuffer(inString);
        for (int i=inString.length(); i < desiredLength; i++)
            newStr.insert(0, padChar);
            
        return newStr.toString();        
    }
    
    // Date in YYMMDD format
    static String simpleCurrentDate()
    {
        // add date of original trade
        Calendar c = Calendar.getInstance();
        int mon = c.get(Calendar.MONTH);
        mon++; // add 1
        int day = c.get(Calendar.DATE);
        int yr = c.get(Calendar.YEAR);
        String sYear = "" + yr;
        if (sYear.length() == 4)
        {
            // reduce to 2 for direct Access format
            sYear = sYear.substring(2);
        }
        String sDate = "" + sYear + ((mon < 10) ? "0" + mon : "" + mon) +
                                    ((day < 10) ? "0" + day : "" + day);
        return sDate;
    }
    
    public static byte [] stringArrayToBuf(String [] strs)
    {
        byte [] buf = null;        
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        OutputStreamWriter osw = new OutputStreamWriter(baos);
        PrintWriter pw = new PrintWriter(osw);
        for (int i=0; i < strs.length; i++)
            pw.println(strs[i]); 
        pw.flush();
        buf = baos.toByteArray();    
        return buf;
    }        

    public static String concat(String [] strs, char delim)
    {
        StringBuffer buf = new StringBuffer();
        
        if (strs == null)
            return null;
            
        for (int i=0; i < strs.length; i++)
        {
            buf.append(strs[i]);
            buf.append(delim);
        }
        
        return buf.toString();
    }
    
    // does this String exist in this array of Strings?
    public static boolean stringExists(String target, String [] set)
    {
        if (set == null) return false;
        
        for (int i=0; i < set.length; i++)
        {
            if (target.equals(set[i]))
            {
                if (debug) System.out.println("target: " + target + " equals " +
                                              set[i]);
                return true;
            }
        }
        
        return false;
    }
    
    // any common elements in these two sets?
    public static boolean intersect(String [] setA, String [] setB)
    {
        boolean result = false;
        
        if (setA == null || setB == null)
            return false;
        
        // any from setA in setB?
        for (int i=0; i < setA.length; i++)
        {
            if (stringExists(setA[i], setB))
            {
                result = true;
                break;
            }
        }
        
        return result;
    }
    
    // does this String exist in this Vector of Strings?
    public static boolean stringExists(String target, Vector set)
    {
        if (set == null) return false;
        
        int setSize = set.size();
        for (int i=0; i < setSize; i++)
        {
            String s = (String) set.elementAt(i);
            if (target.equals(s))
                return true;
        }
        
        return false;
    }
    
    // create a unique set of Strings from a Vector of non-unique
    // strings.
    public static String [] uniqueSet(Vector nonUniqueSet)
    {
        String [] resultSet = null;
        
        if (nonUniqueSet == null)
            return null;
            
        Vector tempVector = new Vector();
        int size = nonUniqueSet.size();
        
        for (int i=0; i < size; i++)
        {
            String next = (String) nonUniqueSet.elementAt(i);
            if (!stringExists(next, tempVector))
                tempVector.addElement(next);
        }
        
        if (tempVector.size() > 0)
        {
            resultSet = new String[tempVector.size()];
            tempVector.copyInto(resultSet);
        }
        
        return resultSet;
    }

    public static void append(Vector allStrings, String [] newStrings)
    {
        if (allStrings == null)
            return;
            
        for (int i = 0; i < newStrings.length; i++)
            allStrings.addElement(newStrings[i]);
    }
    
    public static String quote(String s)
    {
        return "'" + s + "'";   
    }
  
    public static int [] matchIndexes(String word, String buf)
    {
        int [] iarray = null;
        int len = buf.length();
        int idx = 0;
        Vector v = new Vector();
        
        while (idx < len)
        {
            idx = buf.indexOf(word, idx);
            if (idx < 0)
                break;
            v.addElement(new Integer(idx));
            idx++;
        }
        
        if (v.size() > 0)
        {
            iarray = new int[v.size()];
            for (int i=0; i < iarray.length; i++)
                iarray[i] = ((Integer)v.elementAt(i)).intValue();
        }

        return iarray;
    }
  
   public static String [] normalizeXmlValues(String [] vals)
   {
        if (vals == null)
            return null;                
        
        String [] outVals = new String[vals.length];
        for (int i=0; i < vals.length; i++)
            outVals[i] = normalizeXmlValue(vals[i]);
       
        return outVals;
   }
   
    public static String normalizeXmlValue(String value)
    {               
        StringBuffer buf = new StringBuffer(value);
        int size = buf.length();
        int insertPos = -1;
        for (int i=0; i < size; i++)
        {
            char ch = buf.charAt(i);
            switch (ch)
            {
                case '&':
                    buf.setCharAt(i,'&');
                    insertPos = i+1;
                    buf.insert(insertPos, "amp;");
                    size += 4;
                    break;
                case '<':
                    buf.setCharAt(i,'&');
                    insertPos = i+1;
                    buf.insert(insertPos, "lt;");
                    size += 3;
                    break;
                case '>':
                    buf.setCharAt(i,'&');
                    insertPos = i+1;
                    buf.insert(insertPos, "gt;");
                    size += 3;
                    break;
                case '\'':
                    buf.setCharAt(i,'&');
                    insertPos = i+1;
                    buf.insert(insertPos, "apos;");
                    size += 5;
                    break;
                case '"':
                    buf.setCharAt(i,'&');
                    insertPos = i+1;
                    buf.insert(insertPos, "quot;");
                    size += 5;
                    break;
            }
        }
        return buf.toString();
    }   

    public static boolean isBlank(String buf)
    {
        if (buf == null)
            return false;
            
        int len = buf.length();
        for (int i=0; i < len; i++)
        {
            char c = buf.charAt(i);
            if (!Character.isWhitespace(c))
                return false;
        }
        
        return true;
    }

    public static void trimStrings(String [] vals)
    {
        if (vals == null)
            return;
            
        for (int i=0; i < vals.length; i++)
        {
            if (vals[i] != null)
                vals[i] = vals[i].trim();
        }
    }

    public static String removeChar(String input, char c)
    {
        if (input == null)
            return null;
        
        StringBuffer buf = new StringBuffer();
        int count = input.length();
        for (int i=0; i < count; i++)
        {
            char curChar = input.charAt(i);
            if (curChar != c)
                buf.append(curChar);                                        
        }
        return buf.toString();
    }

    public static String trimLetters(String input)
    {
        if (input == null)
            return null;
        
        StringBuffer buf = new StringBuffer();
        int count = input.length();
        for (int i=0; i < count; i++)
        {
            char curChar = input.charAt(i);
            if (Character.isDigit(curChar))
                buf.append(curChar);                                        
        }
        return buf.toString();
    }

    public static String removeString(String source, String target, int limit)
    {
         String result = source;
         int len = target.length();
         int pos = result.indexOf (target);
         int count = 0;

         while ((pos >= 0) && ((limit <= 0) || (count < limit)))
         {
            result = result.substring (0, pos) +
               result.substring (pos + len, result.length());
            count++;
            pos = result.indexOf (target, pos + 1);
         }

         return (result);
      }                

    public static String removeLeadingChar(String input, char c)
    {
        if (input == null)
            return null;
        
        StringBuffer buf = new StringBuffer();
        boolean leading = true;
        
        int count = input.length();
        for (int i=0; i < count; i++)
        {
            char curChar = input.charAt(i);
            if (curChar == c && leading)
                continue;
            else
                leading = false;
                
            buf.append(curChar);                                        
        }
        return buf.toString();
    }

   public static void test ()
   {
        try
        {
         String s = replaceString ("This is a test", "is", "XX", 0);
         System.out.println ("1) " + s);
         s = replaceString ("This is a test", "is", "XX", 1);
         System.out.println ("2) " + s);
         s = replaceString ("This is a test", " ", "< >", 0);
         System.out.println ("3) " + s);
         s = replaceString ("Token", "Token", "Completely replaced", 0);
         System.out.println ("4) " + s);
         System.out.println ("\n");
         String testString = "An expression ( a + b * (c - (a/2)))";
         System.out.println("testString: " + testString);
         int iOpenParen = StringUtil.countOccurences(testString, '(');
         System.out.println("Open parentheses: " + iOpenParen);
         // generate 3 unique id's
         for (int i=0; i < 3; i++)
            System.out.println("#" + i + ": " + uniqueId("Mike", -1));
      
         // get some sequence #'s
         for (int i=0; i < 10; i++)
            System.out.println("seq # " + StringUtil.persistentSequenceNumber("test.dat", 1, 9999));        

         // test tokenize
         String input = "123###4#5###678###hello###wo#rld###9";
         String delim = "###";
         System.out.println("input: " + input);
         System.out.println("delim: " + delim);
         System.out.println("Tokenize with delimiter as single character...");
         String [] toks = tokenize(input, delim);
         for (int i=0; i < toks.length; i++)
                System.out.println("tok[" + i + "]: " + toks[i]);
         System.out.println("Tokenize with delimiter as a group...");
         toks = tokenize(input, delim, true);
         for (int i=0; i < toks.length; i++)
                System.out.println("tok[" + i + "]: " + toks[i]);

         System.out.println("tokenize on a single token, delim as group");
         String input2 = "123";
         toks = tokenize(input2, delim, true);
         for (int i=0; i < toks.length; i++)
                System.out.println("tok[" + i + "]: " + toks[i]);
         
        } catch (Throwable t)
          {
            t.printStackTrace();
          }        
   }

   public static void main (String[] args) // for testing
   {
         test ();
   }
}
