/* XmlUtil.java */
package org.jxul.util;

import java.util.*;

//import GOV.dia.mditds.util.StringUtil;

public class XmlUtil
{
    public static final boolean debug;
    
    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }
    
    public static final String LT = "<";
    public static final String GT = ">";
    public static final String AMP = "&";
    public static final String APOS = "'";
    public static final String QUOT = "\"";
    public static final String EMPTY_TOK = "/" + GT;
    
    public static String [] extractAttributeValues(String xmlBuf, String elemName, boolean empty, String attName)
    {
        String [] values = null;
        
        // first extract the elements
        String [] elements = extractElements(empty, elemName, xmlBuf);
        if (elements != null)
        {
            // iterate through each one and extract the attribute value
            values = new String[elements.length];
            for (int i=0; i < values.length; i++)
                 values[i] = extractAttributeValue(elements[i], attName);
        }
        
        return values;
    }
   
    /*
     * similar to the XSLT normalize-spaces function
     */ 
    public static String normalizeSpaces(String val)
    {
	String retval = "";
	int valuelen = val.length();
	int charno;

        for (charno = 0; charno < valuelen && val.charAt(charno) <= 32; charno++);
	if (charno != valuelen)
		retval = val.substring(charno);

	return (retval.trim());
    }        
    public static String [] extractElements(boolean empty, String elemName, String xmlBuf)
    {
        String [] elements = null;

        Vector v = new Vector(); // TBD: when switch to 1.2, switch to ArrayList

        String startElem = LT + elemName + " ";
        String endElem = LT + "/" + elemName + GT;
        
        int index = 0;
        int bufSize = xmlBuf.length();
        
        if (empty)
        {
            while (index < bufSize)
            {
                int elemStartIdx = xmlBuf.indexOf(startElem, index);
                if (elemStartIdx < 0)
                    break;
                    
                // found one
                // find the end 
                int elemEndIdx = elemStartIdx;
                elemEndIdx = xmlBuf.indexOf(EMPTY_TOK, elemStartIdx);
                if (elemEndIdx < 0)
                    break;
                        
                elemEndIdx += EMPTY_TOK.length();                    
                    
                // store the element
                String elem = xmlBuf.substring(elemStartIdx, elemEndIdx);
                v.addElement(elem);
                
                // reset the index
                index = elemEndIdx + 1;            
            }                            
        }
        else // not empty
        {            
            if (debug) System.out.println("Not empty");
            // find all the starts
            int [] starts = StringUtil.matchIndexes(startElem, xmlBuf);
            if (starts == null)
                return null;

            if (debug)
            {
                System.out.println("# of start indexes: " + starts.length);
                for (int i=0; i < starts.length; i++)
                    System.out.println("start " + i + ":" + starts[i]);
            }
            
            // find all ends
            int [] ends = StringUtil.matchIndexes(endElem, xmlBuf);
            if (ends == null)
                return null;

            if (debug) 
            {
                System.out.println("# of end indexes: " + ends.length);
                for (int i=0; i < ends.length; i++)
                    System.out.println("end " + i + ":" + ends[i]);
            }
                
            if (starts.length != ends.length)
                return null;

            // pair them up
            // 0 is start
            // 1 is end
            int [][] pairs = new int[starts.length][2];                    
            int count = 0;
            for (int i=starts.length - 1; i >= 0; i--)
            {
                if (debug) System.out.println("i is : " + i);
                int aStart = starts[i];
                int theDiff = Integer.MAX_VALUE;
                int theIdx = -1;
                // find the closest end to match
                for (int j=0; j < ends.length; j++)
                {                    
                    if (ends[j] > aStart)    
                    {
                        int diff = ends[j] - aStart;
                        if (diff < theDiff)
                        {                            
                            theDiff = diff;
                            theIdx = j;
                        }
                    }
                } // end of for j
                
                if (theIdx == -1)
                    return null;
                
                if (debug) System.out.println("the closest diff is : " + theDiff + ", index is " + theIdx);
                pairs[count][0] = aStart;
                pairs[count][1] = ends[theIdx];
                count++;
                ends[theIdx] = -1;  // cross this off
            } // end of for i
                              

            for (int i=0; i < pairs.length; i++)
            {
                // store the element
                String elem = xmlBuf.substring(pairs[i][0], pairs[i][1] + endElem.length());
                v.addElement(elem);                
            }        
        } // end of if empty        
        
        if (v.size() > 0)
        {
            elements = new String[v.size()];
            v.copyInto(elements);
        }
        
        return elements;
    }
    
    public static String extractAttributeValue(String element, String attName)
    {
        String value = null;

        // find the attribute
        int attIdx = element.indexOf(attName);
        if (attIdx < 0) return null;
        
        boolean singleQuote = true;
        int firstQuoteIdx = element.indexOf('\'', attIdx);
        
        if (firstQuoteIdx < 0)
        {
            singleQuote = false;
            firstQuoteIdx = element.indexOf('"', attIdx);
            if (firstQuoteIdx < 0)
                return null;
        }
        
        int endQuoteIdx = (singleQuote) ? element.indexOf('\'', firstQuoteIdx + 1) : 
                                          element.indexOf('"', firstQuoteIdx + 1);
        
        value = element.substring(firstQuoteIdx+1, endQuoteIdx);
        
        return value;
    }
    
    public static void main(String [] args)
    {
        String tst1 = "<?xml version = '1.0' ?>" +
                      "<bookmark_list>" +
                      " <folder name='11'>" +
                      "  <bookmark name='2' href='http://www.mike.com' />" +
                      "  <folder name='22'>" +
                      "     <bookmark name='3' href='http://www.mike2.com' />" +
                      "     <bookmark name='4' href='http://www.bob.com' />" +
                      "  </folder>" +
                      "  <boomark name='5' href='http://www.willie.com' />" +
                      "  <folder name='33'>" +
                      "     <bookmark name='46' href='http://www.wiley.com' />" +
                      "  </folder>" +                         
                      " </folder>" +
                      "</bookmark_list>";
        String [] elems1 = extractElements(false, "folder", tst1);
        if (elems1 != null)
        {
            for (int i=0; i < elems1.length; i++)
                System.out.println(i + ")" + elems1[i]);
        }
        else
            System.out.println("No folder elements!");
            
        String [] elems2 = extractElements(true, "bookmark", tst1);
        if (elems2 != null)
        {
            for (int i=0; i < elems2.length; i++)
                System.out.println(i + ")" + elems2[i]);
        }
        else
            System.out.println("No folder elements!");   
            
        System.out.println("*** extract the href's from the bookmarks");
        String [] vals = extractAttributeValues(tst1, "bookmark", true, "href");
        if (vals != null)
        {
            for (int i=0; i < vals.length; i++)
                System.out.println("vals[" + i + "]:" + vals[i]);
        }
        else
            System.out.println("No href attributes!");       
    }
}
